/*
 * Worldvisions Weaver Software:
 *   Copyright (C) 1998, 1999 Worldvisions Computer Technology, Inc.
 *
 * Definition of the WvFastForward class, which listens for connections on a 
 * given set of IP addresses and ports, and redirects them to other locations,
 * based on data in the config file.  For example:
 * 
 * 	[Fast Forward]
 *	aminal:4250 = insight:25
 *
 * will (if running on aminal) accept connections to port 4250 of aminal, and
 * pipe them to insight's sendmail port.
 *
 * If 0.0.0.0 is specified as the "source" IP address, then WvFastForward will
 * listen for connections to port 4250 (in this case) of whatever IP addresses
 * it's able to.
 *
 * Fast Forward also forwards UDP packets.  Currently there is no way to 
 * specify whether you want TCP or UDP -- it always does both.  Fix that.
 *
 */

#ifndef __WVFASTFWD_H
#define __WVFASTFWD_H

#include "wvstreamlist.h"
#include "wvconf.h"
#include "wvipaliaser.h"
#include "wvipfirewall.h"
#include "wvresolver.h"

#include "wvffudp.h"
#include "wvfftcp.h"

#define TCP_XMIT_TIMEOUT	600	// 600 seconds = 10 minutes


extern bool make_port_addr( WvIPPortAddr& addr, const char * str,
			    char ** err );

class WvFastForward : public WvStreamList
/***************************************/
{
public:
    WvFastForward( WvConf& conf );
    virtual ~WvFastForward()
    	{ shutting_down = true; }	// see comment in wvffudp.cc
    bool 	shutting_down;

    void	clean_up();

    void	disable();
    void	reload_config();
    bool	binding_errors() const
    	{ return( broken_bind ); }

    virtual bool select_setup(SelectInfo &si); 

    WvLog log;
    WvLog err;

private:	// FIXME: log and err should probably be private too.
    WvStreamList	listeners;
    WvStreamList	udp_streams;
    WvStreamList	tcp_streams;

    WvConf *		cfg;
    WvIPAliaser		ipalias;
    WvIPFirewall	firewall;

    bool start_listeners();

    bool ok;
    bool broken_bind;

    bool start_tcp_listener( const WvIPPortAddr& src,
    			     const char *	 dest,
    			     bool		 transproxy );
    bool start_udp_listener( const WvIPPortAddr& src,
    			     const char * 	 dest );

public:
    WvStreamList& get_tcp()
    	{ return( tcp_streams ); }
    WvStreamList& get_udp()
    	{ return( udp_streams ); }

    bool too_many_conns() const;
};

#endif __WVFASTFWD_H
