/*
 * Worldvisions Weaver Software:
 *   Copyright (C) 1998, 1999 Worldvisions Computer Technology, Inc.
 *
 * The TCP-related parts of Fast Forward.
 *
 */

#ifndef __WVFFTCP_H
#define __WVFFTCP_H

#include "wvfastfwd.h"
#include "wvhashtable.h"
#include "wvtcp.h"

class WvFF_TCPConn;

class WvFF_TCPListener : public WvTCPListener
/*******************************************/
// A WvFF_TCPListener is a lot like a WvTCPListener, except it also keeps 
// track of what port it's supposed to be forwarding to.
{
public:
    WvFF_TCPListener( const WvIPPortAddr& l_port,
    		      WvFastForward *	  forward,
		      const WvString& 	  dest,
		      bool		  transparent = false );

    WvFF_TCPConn * accept_ff();

private:
    WvFastForward& fwd;
    WvString	   dest_str;

    bool	   transproxy;

    static Callback tcp_accept_callback;
};

class WvFF_TCPConn : public WvTCPConn
/***********************************/
// A WvFF_TCPConn is a lot like a WvTCPConn, except it also keeps track of
// the "other side" of the connection, the last transmit time, and the pending
// write buffer.
{
    friend class WvFF_TCPListener;

protected:
    WvFF_TCPConn( int   open_fd, const WvIPPortAddr& rem );

public:
    WvFF_TCPConn( const WvIPPortAddr& rem, WvFF_TCPConn * rem_conn );
    virtual ~WvFF_TCPConn();

    time_t	   last_xmit;
    WvBuffer	   write_buf;
    WvFF_TCPConn * remote_conn;

    static unsigned int num_conns()
    	{ return( conn_count ); }

private:
    static Callback tcp_callback;

    static unsigned int conn_count;
    static void inc_num_conns()
    	{ conn_count++; }
    static void dec_num_conns()
    	{ conn_count--; }
};

DeclareWvList( WvFF_TCPConn );

#endif __WVFFTCP_H
